import axios from 'axios';
import { config } from '../../config';
const base = "http://localhost:" + config.internal_port

let access_token;
let axios_config;

beforeAll(async () => {
	jest.setTimeout(20000);
	const res = await axios.post(base + '/api/auth/login', { username: "demo", password: "demo" });
	access_token = res.data["access_token"];
	axios_config = {
		headers: { "authorization": "Bearer " + access_token },
		validateStatus: undefined
	};
});

describe('POST /api/donations illegally', () => {
	it('posting to a non-existant endpoint should return 4040', async () => {
		const res1 = await axios.post(base + '/api/donations', null, axios_config);
		expect(res1.status).toEqual(404);
	});
});
// ---------------
describe('POST /api/donations/fixed illegally', () => {
	let added_donor;
	it('creating a new donor with only needed params should return 200', async () => {
		const res = await axios.post(base + '/api/donors', {
			"firstname": "first",
			"lastname": "last"
		}, axios_config);
		added_donor = res.data
		expect(res.status).toEqual(200);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('no input should return 400', async () => {
		const res = await axios.post(base + '/api/donations/fixed', null, axios_config);
		expect(res.status).toEqual(400);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('no donor should return 400', async () => {
		const res = await axios.post(base + '/api/donations/fixed', {
			"amount": 100
		}, axios_config);
		expect(res.status).toEqual(400);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('no amount should return 400', async () => {
		const res = await axios.post(base + '/api/donations/fixed', {
			"donor": added_donor.id
		}, axios_config);
		expect(res.status).toEqual(400);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('illegal amount input should return 400', async () => {
		const res = await axios.post(base + '/api/donations/fixed', {
			"donor": added_donor.id,
			"amount": -1
		}, axios_config);
		expect(res.status).toEqual(400);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('invalid donor input should return 404', async () => {
		const res = await axios.post(base + '/api/donations/fixed', {
			"donor": 999999999999999999999999,
			"amount": 100
		}, axios_config);
		expect(res.status).toEqual(404);
		expect(res.headers['content-type']).toContain("application/json")
	});
});
// ---------------
describe('POST /api/donations/distance illegally', () => {
	let added_donor;
	let added_org;
	let added_runner;
	it('creating a new donor with only needed params should return 200', async () => {
		const res = await axios.post(base + '/api/donors', {
			"firstname": "first",
			"lastname": "last"
		}, axios_config);
		added_donor = res.data
		expect(res.status).toEqual(200);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('creating a new org with just a name should return 200', async () => {
		const res = await axios.post(base + '/api/organizations', {
			"name": "test123"
		}, axios_config);
		added_org = res.data
		expect(res.status).toEqual(200);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('creating a new runner with only needed params should return 200', async () => {
		const res = await axios.post(base + '/api/runners', {
			"firstname": "first",
			"lastname": "last",
			"group": added_org.id
		}, axios_config);
		added_runner = res.data;
		expect(res.status).toEqual(200);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('no input should return 400', async () => {
		const res = await axios.post(base + '/api/donations/distance', null, axios_config);
		expect(res.status).toEqual(400);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('no donor should return 400', async () => {
		const res = await axios.post(base + '/api/donations/distance', {
			"runner": added_runner.id,
			"amountPerDistance": 100,
		}, axios_config);
		expect(res.status).toEqual(400);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('no amountPerDistance should return 400', async () => {
		const res = await axios.post(base + '/api/donations/distance', {
			"runner": added_runner.id,
			"donor": added_donor.id
		}, axios_config);
		expect(res.status).toEqual(400);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('no runner should return 400', async () => {
		const res = await axios.post(base + '/api/donations/distance', {
			"amountPerDistance": 100,
			"donor": added_donor.id
		}, axios_config);
		expect(res.status).toEqual(400);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('illegal amountPerDistance input should return 400', async () => {
		const res = await axios.post(base + '/api/donations/distance', {
			"runner": added_runner.id,
			"amountPerDistance": -1,
			"donor": added_donor.id
		}, axios_config);
		expect(res.status).toEqual(400);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('invalid donor input should return 404', async () => {
		const res = await axios.post(base + '/api/donations/distance', {
			"donor": 999999999999999999999999,
			"runner": added_runner.id,
			"amountPerDistance": 100,
		}, axios_config);
		expect(res.status).toEqual(404);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('invalid runner input should return 404', async () => {
		const res = await axios.post(base + '/api/donations/distance', {
			"donor": added_donor.id,
			"runner": 999999999999999999999999,
			"amountPerDistance": 100,
		}, axios_config);
		expect(res.status).toEqual(404);
		expect(res.headers['content-type']).toContain("application/json")
	});
});
// ---------------
describe('POST /api/donations/fixed successfully', () => {
	let added_donor;
	it('creating a new donor with only needed params should return 200', async () => {
		const res = await axios.post(base + '/api/donors', {
			"firstname": "first",
			"lastname": "last"
		}, axios_config);
		added_donor = res.data
		expect(res.status).toEqual(200);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('creating a new fixed donation with more params should return 200', async () => {
		const res = await axios.post(base + '/api/donations/fixed', {
			"donor": added_donor.id,
			"amount": 1000
		}, axios_config);
		delete res.data.id;
		expect(res.status).toEqual(200);
		expect(res.headers['content-type']).toContain("application/json");
		expect(res.data).toEqual({
			"donor": added_donor,
			"amount": 1000,
			"paidAmount": 0,
			"status": "OPEN",
			"responseType": "DONATION"
		});
	});
	it('creating a new fixed donation with all params should return 200', async () => {
		const res = await axios.post(base + '/api/donations/fixed', {
			"donor": added_donor.id,
			"amount": 1000,
			"paidAmount": 1000
		}, axios_config);
		delete res.data.id;
		expect(res.status).toEqual(200);
		expect(res.headers['content-type']).toContain("application/json");
		expect(res.data).toEqual({
			"donor": added_donor,
			"amount": 1000,
			"paidAmount": 1000,
			"status": "PAID",
			"responseType": "DONATION"
		});
	});
});
// ---------------
describe('POST /api/donations/distance successfully', () => {
	let added_donor;
	let added_org;
	let added_runner;
	let added_donation;
	it('creating a new donor with only needed params should return 200', async () => {
		const res = await axios.post(base + '/api/donors', {
			"firstname": "first",
			"lastname": "last"
		}, axios_config);
		added_donor = res.data
		expect(res.status).toEqual(200);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('creating a new org with just a name should return 200', async () => {
		const res = await axios.post(base + '/api/organizations', {
			"name": "test123"
		}, axios_config);
		added_org = res.data
		expect(res.status).toEqual(200);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('creating a new runner with only needed params should return 200', async () => {
		const res = await axios.post(base + '/api/runners', {
			"firstname": "first",
			"lastname": "last",
			"group": added_org.id
		}, axios_config);
		delete res.data.group;
		added_runner = res.data;
		expect(res.status).toEqual(200);
		expect(res.headers['content-type']).toContain("application/json")
	});
	it('creating a new distance donation with most params should return 200', async () => {
		const res = await axios.post(base + '/api/donations/distance', {
			"runner": added_runner.id,
			"amountPerDistance": 100,
			"donor": added_donor.id
		}, axios_config);
		delete res.data.id;
		expect(res.status).toEqual(200);
		expect(res.headers['content-type']).toContain("application/json");
		expect(res.data).toEqual({
			"donor": added_donor,
			"amountPerDistance": 100,
			"runner": added_runner,
			"amount": 0,
			"paidAmount": 0,
			"status": "PAID",
			"responseType": "DISTANCEDONATION"
		})
	});
	it('creating a new distance donation with all params should return 200', async () => {
		const res = await axios.post(base + '/api/donations/distance', {
			"runner": added_runner.id,
			"amountPerDistance": 100,
			"donor": added_donor.id,
			"paidAmount": 1000
		}, axios_config);
		delete res.data.id;
		expect(res.status).toEqual(200);
		expect(res.headers['content-type']).toContain("application/json");
		expect(res.data).toEqual({
			"donor": added_donor,
			"amountPerDistance": 100,
			"runner": added_runner,
			"amount": 0,
			"paidAmount": 1000,
			"status": "PAID",
			"responseType": "DISTANCEDONATION"
		})
	});
});